// great inspiration taken from https://github.com/jprochazk/garde/blob/main/garde/src/rules/email.rs
use regex::Regex;

use super::AsyncValidateError;

pub fn email_is_valid(email: &str) -> Result<(), AsyncValidateError> {
    if email.is_empty() {
        return Err(AsyncValidateError::new("E-Mail ist leer!"));
    }

    let (user, domain) = email
        .split_once('@')
        .ok_or(AsyncValidateError::new("E-Mail enthält kein '@'!"))?;

    if user.len() > 64 {
        return Err(AsyncValidateError::new("Nutzerteil der E-Mail zu lang!"));
    }

    let user_re = Regex::new(r"(?i-u)^[a-z0-9.!#$%&'*+/=?^_`{|}~-]+\z").unwrap();

    if !user_re.is_match(user) {
        return Err(AsyncValidateError::new(
            "Nutzerteil der E-Mail enthält unerlaubte Zeichen.",
        ));
    }

    if domain.len() > 255 {
        return Err(AsyncValidateError::new(
            "Domainteil der E-Mail ist zu lang.",
        ));
    }

    let domain_re = Regex::new(
        r"(?i-u)^[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?)*$",
    )
    .unwrap();

    if !domain_re.is_match(domain) {
        return Err(AsyncValidateError::new(
            "Domainteil der E-Mail enthält unerlaubte Zeichen!",
        ));
    }

    Ok(())
}

#[test]
pub fn email_validation_works_correctly() {
    assert!(email_is_valid("abc@example.com").is_ok());
    assert!(email_is_valid("admin.new@example-domain.de").is_ok());
    assert!(email_is_valid("admin!new@sub.web.www.example-domain.de").is_ok());

    assert!(email_is_valid("admin.domain.de").is_err());
    assert!(email_is_valid("admin@web@domain.de").is_err());
    assert!(email_is_valid("@domain.de").is_err());
    assert!(email_is_valid("user@").is_err());
    assert!(email_is_valid("").is_err());
}
